from ast import keyword
from cProfile import label
import datetime
from joblib import Parallel
import nltk
from newspaper import Article  # for parsing Reuters articles
import numpy as np
from dataclasses import dataclass
import requests
import csv
import pickle
import json
import os
from joblib import Parallel, delayed  # for parallel processing
from tqdm import tqdm


@dataclass
class NewsStory:
    stock_ticker: str
    title: str
    url: str
    authors: str
    publish_date: str
    text: str
    keywords: str
    summary: str
    label: str

    def __repr__(self):
        return f"Ticker: {self.stock_ticker} \nTitle: {self.title} \nURL: {self.url} \nAuthors: {self.authors} \nPublish Date: {self.publish_date} \nText: {self.text} \nKeywords: {self.keywords} \nSummary: {self.summary}\n"

    def __hash__(self):
        return hash(self.url)


def importNewsFromJSON(file):
    # takes in a json file in the format of:

    # {
    #     "authors": [],
    #     "keywords": [],
    #     "publish_date": "05/21/21",
    #     "stock_ticker": "",
    #     "summary": "",
    #     "text": "",
    #     "title": "",
    #     "url": ""
    # },

    # returns a list of NewsStory objects

    print(f"Importing {file}")
    newsStories = []
    with open(file, 'r') as jsonfile:
        data = json.load(jsonfile)
        for row in data:

            # if label exists
            if "label" in row:
                label = row["label"]
            else:
                label = "0"

            newsStories.append(NewsStory(row["stock_ticker"],
                                         row["title"],
                                         row["url"],
                                         row["authors"],
                                         row["publish_date"],
                                         row["text"],
                                         row["keywords"],
                                         row["summary"],
                                         label))
    return newsStories


def importAllNewsFromJSON(directory):
    # takes in a directory of json files
    # returns a list of NewsStory objects

    newsStories = []
    for file in os.listdir(directory):
        if file.endswith(".json"):
            newsStories.extend(importNewsFromJSON(directory + "/" + file))
    return newsStories


def writeNewsToJSON(newsStories, file_name):

    file_name = "./data_labeld/" + file_name + ".json"
    with open(file_name, 'w') as outfile:
        outfile.write(json.dumps(
            newsStories, default=lambda o: o.__dict__, sort_keys=True, indent=4))


def lookForMatch(newsStory, string):
    # takes in a NewsStory objects and a string
    # returns the NewsStory objects that contains the string case-insensitively

    newsStories = []
    string = string.lower()
    for news in newsStory:
        if string in news.title.lower() or string in news.text.lower() or string in news.summary.lower() or string in news.keywords:
            if news.label == "0":
                newsStories.append(news)

    return newsStories


def combineData():
    print("Loading all news stories in ./reuters_data_enriched/")
    news = importAllNewsFromJSON("./reuters_data_enriched")
    lenNews = len(news)
    print("Imported {} news stories".format(lenNews))
    #writeNewsToJSON(news, "all_news_enriched")

    news = sorted(news, key=lambda x: x.stock_ticker)
    # write to a json file for each ticker
    for ticker in tqdm(set([news.stock_ticker for news in news])):
        ticker_news = [news for news in news if news.stock_ticker == ticker]
        ticker_news = removeDuplicates(ticker_news)
        print(f"Writing news stories to ./data_labeld/{ticker}.json")
        writeNewsToJSON(ticker_news, ticker)


def shortPrint(newsStories):
    # takes in a list of NewsStory objects
    # prints the title, url, and publish date of each news story

    # sort by ticker
    newsStories = sorted(newsStories, key=lambda x: x.stock_ticker)

    for news in newsStories:
        print(
            f"Title: {news.title} \nURL: {news.url} \nTicker: {news.stock_ticker} \nPublish Date: {news.publish_date}\n")


def removeDuplicates(newsStories):
    # takes in a list of NewsStory objects
    # returns a list of NewsStory objects
    # with duplicate text removed
    # keeps the oldest news story

    startLenght = len(newsStories)
    print(f"Number of news stories before removing duplicates: {startLenght}")
    print("Removing duplicates...")

    # sort by publish date
    newsStories = sorted(newsStories, key=lambda x: x.publish_date)

    # remove duplicates by checking text
    text1 = []
    newsStories2 = []
    for news in newsStories:
        if news.text not in text1:
            text1.append(news.text)
            newsStories2.append(news)

    print(
        f"Number of news stories after removing duplicates: {len(newsStories2)}")
    print(f"Removed {startLenght - len(newsStories2)} news stories")

    return newsStories2


if __name__ == "__main__":

    # reads in all news stories from ./reuters_data_enriched/, removes duplicates, and writes them to ./data_labeld/ticker.json with label 0
    # combineData()

    news = importAllNewsFromJSON("./data_labeld")
    print("\nImported {} news stories\n".format(len(news)))

    # done: "Cyber attack", "Cyberattack", "data breach", "ransomware", "hackers", "security flaw", and "gained access"
    searchString = "Cyberattack"
    newsStories = lookForMatch(news, searchString)
    shortPrint(newsStories)
    
    # print the number of news stories with label 1
    count = 0
    for newsStory in news:
        if newsStory.label == "1":
            count += 1
    print(f"Number of news stories with label 1: {count}")
